#: The default alphabet used for generating nanoid strings.
ALPHABET = "_-0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"

#: The default size of the generated nanoid strings.
SIZE = 21
